@@ -4,8 +4,8 @@ lightweight wrapper around Redis.
 
 """
 from collections import OrderedDict
-import datetime
-import json
+from datetime import datetime, timedelta
+
 import redis
 
 from django.conf import settings
@@ -67,27 +67,71 @@ class R(object):
             getattr(settings, 'REDIS_METRICS_SOCKET_CONNECTION_POOL', None))
 
         # Create the connection to Redis
-        self.r = redis.StrictRedis(host=self.host, port=self.port, db=self.db,
-                                   password=self.password,
-                                   socket_timeout=self.socket_timeout,
-                                   connection_pool=self.connection_pool,
-                                   decode_responses=True)
+        self.r = redis.StrictRedis(
+            host=self.host,
+            port=self.port,
+            db=self.db,
+            password=self.password,
+            socket_timeout=self.socket_timeout,
+            connection_pool=self.connection_pool,
+            decode_responses=True
+        )
 
-    def _date_range(self, since=None):
+    def _date_range(self, granularity, since):
         """Returns a generator that yields ``datetime.datetime`` objects from
-        the ``since`` date until *now*. If ``since`` is omitted, returns dates
-        for one year.
+        the ``since`` date until *now*.
 
-        * ``since`` -- a ``datetime.datetime`` object or None.
+        * ``granularity`` -- The granularity at which the generated datetime
+          objects should be created: seconds, minutes, hourly, daily, weekly,
+          monthly, or yearly
+        * ``since`` -- a ``datetime.datetime`` object, from which we start
+          generating periods of time. This can also be ``None``, and will
+          default to the past 7 days if that's the case.
+
+        If ``granularity`` is one of daily, weekly, monthly, or yearly, this
+        function gives objects at the daily level.
+
+        If ``granularity`` is one of the following, the number of datetime
+        objects returned is capped, otherwise this code is really slow and
+        probably generates more data than we want:
+
+            * hourly: returns at most 720 values (~30 days)
+            * minutes: returns at most 480 values (8 hours)
+            * second: returns at most 300 values (5 minutes)
+
+        For example, if granularity is "seconds", we'll receive datetime
+        objects that differ by 1 second each.
 
         """
-        now = datetime.datetime.today()
         if since is None:
-            since_days = 365  # assume 1 year :-/
+            since = datetime.utcnow() - timedelta(days=7)  # Default to 7 days
+
+        now = datetime.utcnow()
+        elapsed = (now - since)
+
+        # Figure out how many units to generate for the elapsed time.
+        # I'm going to use `granularity` as a keyword parameter to timedelta,
+        # so I need to change the wording for hours and anything > days.
+        if granularity == "seconds":
+            units = elapsed.total_seconds()
+            units = 300 if units > 300 else units
+        elif granularity == "minutes":
+            units = elapsed.total_seconds() / 60
+            units = 480 if units > 480 else units
+        elif granularity == "hourly":
+            granularity = "hours"
+            units = elapsed.total_seconds() / 3600
+            units = 720 if units > 720 else units
         else:
-            delta = now - since  # The timedelta from "since" to "now"
-            since_days = delta.days + 1
-        return (now - datetime.timedelta(days=d) for d in range(since_days))
+            granularity = "days"
+            units = elapsed.days + 1
+
+        return (now - timedelta(**{granularity: u}) for u in range(int(units)))
+
+    def categories(self):
+        """Returns a set of Categories under which metrics may have been
+        organized."""
+        return self.r.smembers(self._categories_key)
 
     def _category_key(self, category):
         return u"c:{0}".format(category)
@@ -119,21 +163,25 @@ class R(object):
         """Builds redis keys used to store metrics.
 
         * ``slug`` -- a slug used for a metric, e.g. "user-signups"
-        * ``date`` -- (optional) A ``datetime.date`` or ``datetime.datetime``
-          objects used to generate the time period for the metric. If omitted,
-          the current date will be used.
-        * ``granularity`` -- Must be one of: "all" (default), "daily",
-          "weekly", "monthly", "yearly".
+        * ``date`` -- (optional) A ``datetime.datetime`` object used to
+          generate the time period for the metric. If omitted, the current date
+          and time (in UTC) will be used.
+        * ``granularity`` -- Must be one of: "all" (default), "yearly",
+        "monthly", "weekly", "daily", "hourly", "minutes", or "seconds".
 
         Returns a list of strings.
 
         """
-        slug = slugify(slug)  # Make sure our slugs have a consistent format
+        slug = slugify(slug)  # Ensure slugs have a consistent format
         if date is None:
-            date = datetime.date.today()
+            date = datetime.utcnow()
 
-        # we want to keep the order, here: daily, weekly, monthly, yearly
+        # we want to keep the order, here, from smallest to largest granularity:
+        # seconds, minutes, hourly, daily, weekly, monthly, yearly
         patts = OrderedDict()
+        patts["seconds"] = "m:{0}:s:{1}".format(slug, date.strftime("%Y-%m-%d-%H-%M-%S"))
+        patts["minutes"] = "m:{0}:i:{1}".format(slug, date.strftime("%Y-%m-%d-%H-%M"))
+        patts["hourly"] = "m:{0}:h:{1}".format(slug, date.strftime("%Y-%m-%d-%H"))
         patts["daily"] = "m:{0}:{1}".format(slug, date.strftime("%Y-%m-%d"))
         patts["weekly"] = "m:{0}:w:{1}".format(slug, date.strftime("%Y-%U"))
         patts["monthly"] = "m:{0}:m:{1}".format(slug, date.strftime("%Y-%m"))
@@ -147,8 +195,7 @@ class R(object):
     def metric_slugs(self):
         """Return a set of metric slugs (i.e. those used to create Redis keys)
         for this app."""
-        keys = self.r.smembers(self._metric_slugs_key)
-        return set(s.split(":")[1] for s in keys)
+        return self.r.smembers(self._metric_slugs_key)
 
     def metric_slugs_by_category(self):
         """Return a dictionary of category->metrics data:
@@ -176,24 +223,66 @@ class R(object):
         """Removes all keys for the given ``slug``."""
 
         # To remove all keys for a slug, I need to retrieve them all from
-        # the set of metric keys, then filter the matching prefixes.
-        # This is quite possibly the most inefficient way to do this :-/
-        prefix = "m:{0}:".format(slug)
-        keys = [
-            k for k in self.r.smembers(self._metric_slugs_key)
-            if k.startswith(prefix)
-        ]
+        # the set of metric keys, This uses the redis "keys" command, which is
+        # inefficient, but this shouldn't be used all that often.
+        prefix = "m:{0}:*".format(slug)
+        keys = self.r.keys(prefix)
+        self.r.delete(*keys)  # Remove the metric data
 
-        # Remove the metric data
-        self.r.delete(*keys)
+        # Finally, remove the slug from the set
+        self.r.srem(self._metric_slugs_key, slug)
 
-        # Finally, remove keys from the set
-        self.r.srem(self._metric_slugs_key, *keys)
+    def set_metric(self, slug, value, category=None, expire=None):
+        """Assigns a specific value to the *current* metric. You can use this
+        to start a metric at a value greater than 0 or to reset a metric.
+
+        The given slug will be used to generate Redis keys at the following
+        granularities: Seconds, Minutes, Hours, Day, Week, Month, and Year.
+
+        Parameters:
+
+        * ``slug`` -- a unique value to identify the metric; used in
+          construction of redis keys (see below).
+        * ``value`` -- The value of the metric.
+        * ``category`` -- (optional) Assign the metric to a Category (a string)
+        * ``expire`` -- (optional) Specify the number of seconds in which the
+          metric will expire.
+
+        Redis keys for each metric (slug) take the form:
+
+            m:<slug>:s:<yyyy-mm-dd-hh-mm-ss> # Second
+            m:<slug>:i:<yyyy-mm-dd-hh-mm>    # Minute
+            m:<slug>:h:<yyyy-mm-dd-hh>       # Hour
+            m:<slug>:<yyyy-mm-dd>            # Day
+            m:<slug>:w:<yyyy-num>            # Week (year - week number)
+            m:<slug>:m:<yyyy-mm>             # Month
+            m:<slug>:y:<yyyy>                # Year
+
+        """
+        keys = self._build_keys(slug)
+
+        # Add the slug to the set of metric slugs
+        self.r.sadd(self._metric_slugs_key, slug)
+
+        # Construct a dictionary of key/values for use with mset
+        data = {}
+        for k in keys:
+            data[k] = value
+        self.r.mset(data)
+
+        # Add the category if applicable.
+        if category:
+            self._categorize(slug, category)
+
+        # Expire the Metric in ``expire`` seconds if applicable.
+        if expire:
+            for k in keys:
+                self.r.expire(k, expire)
 
     def metric(self, slug, num=1, category=None, expire=None):
         """Records a metric, creating it if it doesn't exist or incrementing it
         if it does. All metrics are prefixed with 'm', and automatically
-        aggregate for Day, Week, Month, and Year.
+        aggregate for Seconds, Minutes, Hours, Day, Week, Month, and Year.
 
         Parameters:
 
@@ -206,20 +295,26 @@ class R(object):
 
         Redis keys for each metric (slug) take the form:
 
-            m:<slug>:<yyyy-mm-dd>   # Day
-            m:<slug>:w:<yyyy-num>   # Week (year - week number)
-            m:<slug>:m:<yyyy-mm>    # Month
-            m:<slug>:y:<yyyy>       # Year
+            m:<slug>:s:<yyyy-mm-dd-hh-mm-ss> # Second
+            m:<slug>:i:<yyyy-mm-dd-hh-mm>    # Minute
+            m:<slug>:h:<yyyy-mm-dd-hh>       # Hour
+            m:<slug>:<yyyy-mm-dd>            # Day
+            m:<slug>:w:<yyyy-num>            # Week (year - week number)
+            m:<slug>:m:<yyyy-mm>             # Month
+            m:<slug>:y:<yyyy>                # Year
 
         """
-        day_key, week_key, month_key, year_key = self._build_keys(slug)
+        keys = self._build_keys(slug)
+        sec_key, min_key, hour_key, day_key, week_key, month_key, year_key = keys
 
-        # Keep track of all of our keys
-        self.r.sadd(self._metric_slugs_key,
-                    day_key, week_key, month_key, year_key)
+        # Add the slug to the set of metric slugs
+        self.r.sadd(self._metric_slugs_key, slug)
 
         # Increment keys. NOTE: current redis-py (2.7.2) doesn't include an
-        # incrby method; .incr accepts a second ``amound`` parameter.
+        # incrby method; .incr accepts a second ``amount`` parameter.
+        self.r.incr(sec_key, num)
+        self.r.incr(min_key, num)
+        self.r.incr(hour_key, num)
         self.r.incr(day_key, num)
         self.r.incr(week_key, num)
         self.r.incr(month_key, num)
@@ -230,6 +325,9 @@ class R(object):
 
         # Expire the Metric in ``expire`` seconds
         if expire:
+            self.r.expire(sec_key, expire)
+            self.r.expire(min_key, expire)
+            self.r.expire(hour_key, expire)
             self.r.expire(day_key, expire)
             self.r.expire(week_key, expire)
             self.r.expire(month_key, expire)
@@ -238,12 +336,16 @@ class R(object):
     def get_metric(self, slug):
         """Get the current values for a metric.
 
-        Returns a dict with metric values accumulated for the day, week, month,
-        and year.
+        Returns a dictionary with metric values accumulated for the seconds,
+        minutes, hours, day, week, month, and year.
 
         """
-        day_key, week_key, month_key, year_key = self._build_keys(slug)
+        keys = self._build_keys(slug)
+        sec_key, min_key, hour_key, day_key, week_key, month_key, year_key = keys
         return {
+            'seconds': self.r.get(sec_key),
+            'minutes': self.r.get(min_key),
+            'hours': self.r.get(hour_key),
             'day': self.r.get(day_key),
             'week': self.r.get(week_key),
             'month': self.r.get(month_key),
@@ -256,15 +358,21 @@ class R(object):
         Returns a list of two-tuples containing the metric slug and a
         dictionary like the one returned by ``get_metric``::
 
-            (some-metric, {'day': 0, 'week': 0, 'month': 0, 'year': 0})
-
-        NOTE: This method calls ``get_metric`` for each slug in the
-        ``slug_list``, which really isn't very efficient.
+            (
+                some-metric, {
+                    'seconds': 0, 'minutes': 0, 'hours': 0,
+                    'day': 0, 'week': 0, 'month': 0, 'year': 0
+                }
+            )
 
         """
         results = []
         for slug in slug_list:
-            results.append((slug, self.get_metric(slug)))
+            keys = ['seconds', 'minutes', 'hours', 'day', 'week', 'month', 'year']
+            metrics = self.r.mget(*self._build_keys(slug))
+            if any(metrics):  # Only if we have data.
+                d = dict(zip(keys, metrics))
+                results.append((slug, d))
         return results
 
     def get_category_metrics(self, category):
@@ -290,15 +398,13 @@ class R(object):
             category.
 
         """
+        key = self._category_key(category)
         if len(metric_slugs) == 0:
             # If there are no metrics, just remove the category
             self.delete_category(category)
         else:
-            # Convert metrics to json (removing any duplicates)
-            json_data = json.dumps(list(metric_slugs))
-            self.r.set(self._category_key(category), json_data)
-
-            # Store all category names in a Redis set, for easy retrieval
+            # Save all the slugs in the category, and save the category name
+            self.r.sadd(key, *metric_slugs)
             self.r.sadd(self._categories_key, category)
 
     def get_metric_history(self, slugs, since=None, granularity='daily'):
@@ -306,7 +412,8 @@ class R(object):
 
         * ``slugs`` -- a slug OR a list of slugs
         * ``since`` -- the date from which we start pulling metrics
-        * ``granularity`` -- daily, weekly, monthly, yearly
+        * ``granularity`` -- seconds, minutes, hourly,
+                             daily, weekly, monthly, yearly
 
         Returns a list of tuples containing the Redis key and the associated
         metric::
@@ -333,9 +440,13 @@ class R(object):
         # Build the set of Redis keys that we need to get.
         keys = set()
         for slug in slugs:
-            for date in self._date_range(since):
+            for date in self._date_range(granularity, since):
                 keys.update(set(self._build_keys(slug, date, granularity)))
-        return sorted(zip(keys, self.r.mget(keys)))
+
+        # Fetch our data, replacing any None-values with zeros
+        results = [0 if v is None else v for v in self.r.mget(keys)]
+        results = zip(keys, results)
+        return sorted(results, key=lambda t: t[0])
 
     def get_metric_history_as_columns(self, slugs, since=None,
                                       granularity='daily'):
@@ -380,17 +491,74 @@ class R(object):
                     periods.append(period)
 
             _history.append(column)  # Remember that slug's column of data
-        _history.insert(0, periods)  # Finally, stick the time periods in the
-                                     # first column.
+
+        # Finally, stick the time periods in the first column.
+        _history.insert(0, periods)
         return zip(*_history)  # Transpose the rows & columns
 
+    def get_metric_history_chart_data(self, slugs, since=None, granularity='daily'):
+        """Provides the same data as ``get_metric_history``, but with metrics
+        data arranged in a format that's easy to plot with Chart.js. If you had
+        the following yearly history, for example::
+
+            [
+                ('m:bar:y:2012', '1'),
+                ('m:bar:y:2013', '2'),
+                ('m:bar:y:2014', '3'),
+                ('m:foo:y:2012', '4'),
+                ('m:foo:y:2013', '5')
+                ('m:foo:y:2014', '6')
+            ]
+
+        this method would provide you with the following data structure::
+
+            'periods': ['y:2012', 'y:2013', 'y:2014']
+            'data': [
+              {
+                'slug': 'bar',
+                'values': [1, 2, 3]
+              },
+              {
+                'slug': 'foo',
+                'values': [4, 5, 6]
+              },
+            ]
+
+        """
+        slugs = sorted(slugs)
+        history = self.get_metric_history(slugs, since, granularity)
+
+        # Convert the history into an intermediate data structure organized
+        # by periods. Since the history is sorted by key (which includes both
+        # the slug and the date, the values should be ordered correctly.
+        periods = []
+        data = OrderedDict()
+        for k, v in history:
+            period = template_tags.strip_metric_prefix(k)
+            if period not in periods:
+                periods.append(period)
+
+            slug = template_tags.metric_slug(k)
+            if slug not in data:
+                data[slug] = []
+            data[slug].append(v)
+
+        # Now, reorganize data for our end result.
+        metrics = {'periods': periods, 'data': []}
+        for slug, values in data.items():
+            metrics['data'].append({
+                'slug': slug,
+                'values': values
+            })
+
+        return metrics  # templates still don't like defaultdict's
+
     # Gauges. Gauges have a different prefix "g:" in order to differentiate
     # them from a metric of the same name.
     def gauge_slugs(self):
         """Return a set of Gauges slugs (i.e. those used to create Redis keys)
         for this app."""
-        keys = self.r.smembers(self._gauge_slugs_key)
-        return set(s.split(":")[1] for s in keys)
+        return self.r.smembers(self._gauge_slugs_key)
 
     def _gauge_key(self, slug):
         """Make sure our slugs have a consistent format."""
@@ -404,7 +572,7 @@ class R(object):
 
         """
         k = self._gauge_key(slug)
-        self.r.sadd(self._gauge_slugs_key, k)  # keep track of all Gauges
+        self.r.sadd(self._gauge_slugs_key, slug)  # keep track of all Gauges
         self.r.set(k, current_value)
 
     def get_gauge(self, slug):
@@ -415,4 +583,4 @@ class R(object):
         """Removes all gauges with the given ``slug``."""
         key = self._gauge_key(slug)
         self.r.delete(key)  # Remove the Gauge
-        self.r.srem(self._gauge_slugs_key, key)  # Remove from the set of keys
+        self.r.srem(self._gauge_slugs_key, slug)  # Remove from the set of keys
