@@ -4,7 +4,7 @@ when you run "manage.py test".
 
 Replace this with more appropriate tests for your application.
 """
-from datetime import date, datetime
+from datetime import datetime, timedelta
 from mock import call, patch, Mock
 
 from django.conf import settings
@@ -107,20 +107,66 @@ class TestR(TestCase):
             mock_redis.assert_called_once_with(**r_kwargs)
 
     def test__date_range(self):
-        """Tests ``R._date_range``."""
+        """Tests ``R._date_range`` at various granularities *without* a
+        ``since`` date."""
+        # minutes, seconds and hours should be capped at 300, 480, and 720
+        d = datetime(2014, 1, 1)
+        self.assertEqual(len(list(self.r._date_range('seconds', d))), 300)
+        self.assertEqual(len(list(self.r._date_range('minutes', d))), 480)
+        self.assertEqual(len(list(self.r._date_range('hourly', d))), 720)
 
-        # Verify that omitting the ``since`` parameter gives you dates for the
-        # previous year.
-        dates = [d for d in self.r._date_range()]
-        self.assertEqual(len(dates), 365)
+        # Everything else should have a timedelta of 7 days
+        self.assertEqual(len(list(self.r._date_range('daily', None))), 8)
+        self.assertEqual(len(list(self.r._date_range('weekly', None))), 8)
+        self.assertEqual(len(list(self.r._date_range('monthly', None))), 8)
+        self.assertEqual(len(list(self.r._date_range('yearly', None))), 8)
 
-        # Provide a ``since`` parameter.
-        t = datetime(2012, 12, 25)  # Merry Christmas!
-        # NOTE: just check for dates, ignoring hours, mins, seconds, etc.
-        dates = [d.date() for d in self.r._date_range(since=t)]
+    def test__date_range_seconds(self):
+        """Tests ``R._date_range`` at the "seconds" granularity."""
+        since = datetime.utcnow() - timedelta(seconds=5)
+        values = self.r._date_range('seconds', since)
+        self.assertEqual(len(list(values)), 5)
 
-        self.assertIn(t.date(), dates)  # Should include our specified date
-        self.assertGreater(len(dates), 1)  # There should be some dates
+    def test__date_range_minutes(self):
+        """Tests ``R._date_range`` at the "minutes" granularity."""
+        since = datetime.utcnow() - timedelta(minutes=5)
+        values = self.r._date_range('minutes', since)
+        self.assertEqual(len(list(values)), 5)
+
+    def test__date_range_hourly(self):
+        """Tests ``R._date_range`` at the "hourly" granularity."""
+        since = datetime.utcnow() - timedelta(hours=5)
+        values = self.r._date_range('hourly', since)
+        self.assertEqual(len(list(values)), 5)
+
+    def test__date_range_daily(self):
+        """Tests ``R._date_range`` at the "daily" granularity."""
+        since = datetime.utcnow() - timedelta(days=5)
+        values = self.r._date_range('daily', since)
+        self.assertEqual(len(list(values)), 6)  # 6 because we do days + 1
+
+    def test__date_range_weekly(self):
+        """Tests ``R._date_range`` at the "weekly" granularity."""
+        since = datetime.utcnow() - timedelta(days=5)
+        values = self.r._date_range('weekly', since)
+        self.assertEqual(len(list(values)), 6)  # 6 because we do days + 1
+
+    def test__date_range_monthly(self):
+        """Tests ``R._date_range`` at the "monthly" granularity."""
+        since = datetime.utcnow() - timedelta(days=5)
+        values = self.r._date_range('montly', since)
+        self.assertEqual(len(list(values)), 6)  # 6 because we do days + 1
+
+    def test__date_range_yearly(self):
+        """Tests ``R._date_range`` at the "yearly" granularity."""
+        since = datetime.utcnow() - timedelta(days=5)
+        values = self.r._date_range('yearly', since)
+        self.assertEqual(len(list(values)), 6)  # 6 because we do days + 1
+
+    def test_categories(self):
+        """Verify that ``R.categories()`` calls redis SMEMBERS command."""
+        self.r.categories()
+        self.redis.smembers.assert_called_once_with("categories")
 
     def test__category_key(self):
         """Creates a redis key for a given category string."""
@@ -165,38 +211,59 @@ class TestR(TestCase):
 
     def test__build_keys(self):
         """Tests ``R._build_keys``. with default arguments."""
-        d = date.today()
-        slug = 'test-slug'
-        expected_results = [
-            "m:{0}:{1}".format(slug, d.strftime("%Y-%m-%d")),
-            "m:{0}:w:{1}".format(slug, d.strftime("%Y-%U")),
-            "m:{0}:m:{1}".format(slug, d.strftime("%Y-%m")),
-            "m:{0}:y:{1}".format(slug, d.strftime("%Y")),
-        ]
-        keys = self.r._build_keys(slug)
-        self.assertEqual(keys, expected_results)
+        with patch('redis_metrics.models.datetime') as mock_datetime:
+            mock_datetime.utcnow.return_value = datetime(2014, 7, 2, 12, 6, 34)
+            expected_results = [
+                "m:test-slug:s:2014-07-02-12-06-34",
+                "m:test-slug:i:2014-07-02-12-06",
+                "m:test-slug:h:2014-07-02-12",
+                "m:test-slug:2014-07-02",
+                "m:test-slug:w:2014-26",
+                "m:test-slug:m:2014-07",
+                "m:test-slug:y:2014",
+            ]
+            keys = self.r._build_keys('test-slug')
+            self.assertEqual(keys, expected_results)
+
+    def test__build_keys_seconds(self):
+        """Tests ``R._build_keys``. with a *seconds* granularity."""
+        d = datetime(2012, 4, 1, 11, 30, 59)  # April Fools!
+        keys = self.r._build_keys('test-slug', date=d, granularity='seconds')
+        self.assertEqual(keys, ['m:test-slug:s:2012-04-01-11-30-59'])
+
+    def test__build_keys_minutes(self):
+        """Tests ``R._build_keys``. with a *minutes* granularity."""
+        d = datetime(2012, 4, 1, 11, 30)  # April Fools!
+        keys = self.r._build_keys('test-slug', date=d, granularity='minutes')
+        self.assertEqual(keys, ['m:test-slug:i:2012-04-01-11-30'])
+
+    def test__build_keys_hourly(self):
+        """Tests ``R._build_keys``. with a *hourly* granularity."""
+        d = datetime(2012, 4, 1, 11, 30)  # April Fools!
+        keys = self.r._build_keys('test-slug', date=d, granularity='hourly')
+        self.assertEqual(keys, ['m:test-slug:h:2012-04-01-11'])
 
     def test__build_keys_daily(self):
         """Tests ``R._build_keys``. with a *daily* granularity."""
-        d = date(2012, 4, 1)  # April Fools!
+        d = datetime(2012, 4, 1)  # April Fools!
         keys = self.r._build_keys('test-slug', date=d, granularity='daily')
         self.assertEqual(keys, ['m:test-slug:2012-04-01'])
 
     def test__build_keys_weekly(self):
         """Tests ``R._build_keys``. with a *weekly* granularity."""
-        d = date(2012, 4, 1)  # April Fools!
+        d = datetime(2012, 4, 1)  # April Fools!
         keys = self.r._build_keys('test-slug', date=d, granularity='weekly')
         self.assertEqual(keys, ['m:test-slug:w:2012-14'])
 
     def test__build_keys_monthly(self):
         """Tests ``R._build_keys``. with a *monthly* granularity."""
-        d = date(2012, 4, 1)  # April Fools!
+        d = datetime(2012, 4, 1)  # April Fools!
         keys = self.r._build_keys('test-slug', date=d, granularity='monthly')
         self.assertEqual(keys, ['m:test-slug:m:2012-04'])
 
     def test__build_keys_yearly(self):
         """Tests ``R._build_keys``. with a *yearly* granularity."""
-        d = date(2012, 4, 1)  # April Fools!
+        d = datetime(2012, 4, 1)  # April Fools!
         keys = self.r._build_keys('test-slug', date=d, granularity='yearly')
         self.assertEqual(keys, ['m:test-slug:y:2012'])
 
@@ -233,17 +300,91 @@ class TestR(TestCase):
         """Verify that ``R.delete_metric`` deletes all keys and removes keys
         from the set of metric slugs."""
 
-        # Make sure SMEMBERS returns some data
-        self.redis.smembers.return_value = ["m:slug:0", "m:slug:1"]
+        # Make sure KEYS returns some data
+        self.redis.keys.return_value = ["m:slug:0", "m:slug:1"]
         self.r.delete_metric('slug')  # call delete_metric
 
         # Verify that the metric data is removed as are the keys from the set
         self.redis.assert_has_calls([
-            call.smembers(self.r._metric_slugs_key),
+            call.keys("m:slug:*"),
             call.delete("m:slug:0", "m:slug:1"),
-            call.srem(self.r._metric_slugs_key, "m:slug:0", "m:slug:1"),
+            call.srem(self.r._metric_slugs_key, "slug")
         ])
 
+    @patch.object(R, '_build_keys')
+    def test_set_metric(self, mock_build_keys):
+        """Test setting metrics using ``R.set_metric``."""
+        # Define redis keys
+        mock_build_keys.return_value = [
+            'm:test-slug:s:2000-01-02-11-45-30',
+            'm:test-slug:i:2000-01-02-11-45',
+            'm:test-slug:h:2000-01-02-11',
+            'm:test-slug:2000-01-02',
+            'm:test-slug:m:2000-01',
+            'm:test-slug:w:2000-01',
+            'm:test-slug:y:2000'
+        ]
+        slug = 'test-slug'
+        value = 42
+
+        # get the metric keys so we can check for the appropriate calls
+        self.r.set_metric(slug, value)
+
+        # Verify that setting a metric adds the appropriate slugs to the keys
+        # set and then incrememts each key
+        self.redis.sadd.assert_called_once_with('metric-slugs', slug)
+        self.redis.mset.assert_called_once_with({
+            'm:test-slug:s:2000-01-02-11-45-30': 42,
+            'm:test-slug:i:2000-01-02-11-45': 42,
+            'm:test-slug:h:2000-01-02-11': 42,
+            'm:test-slug:2000-01-02': 42,
+            'm:test-slug:m:2000-01': 42,
+            'm:test-slug:w:2000-01': 42,
+            'm:test-slug:y:2000': 42,
+        })
+
+        # Expiration should not have gotten called
+        self.assertFalse(self.redis.expire.called)
+
+    @patch.object(R, '_build_keys')
+    def test_set_metric_with_expiration(self, mock_build_keys):
+        """Test setting metrics using ``R.set_metric`` with an expiration."""
+        # Define redis keys
+        mock_build_keys.return_value = [
+            'm:test-slug:s:2000-01-02-11-45-30',
+            'm:test-slug:i:2000-01-02-11-45',
+            'm:test-slug:h:2000-01-02-11',
+            'm:test-slug:2000-01-02',
+            'm:test-slug:m:2000-01',
+            'm:test-slug:w:2000-01',
+            'm:test-slug:y:2000'
+        ]
+        slug = 'test-slug'
+        value = 42
+
+        # get the metric keys so we can check for the appropriate calls
+        self.r.set_metric(slug, value, expire=500)
+
+        # Verify that each key had an expiration set.
+        self.redis.expire.assert_has_calls([
+            call('m:test-slug:s:2000-01-02-11-45-30', 500),
+            call('m:test-slug:i:2000-01-02-11-45', 500),
+            call('m:test-slug:h:2000-01-02-11', 500),
+            call('m:test-slug:2000-01-02', 500),
+            call('m:test-slug:m:2000-01', 500),
+            call('m:test-slug:w:2000-01', 500),
+            call('m:test-slug:y:2000', 500),
+        ])
+
+    @patch.object(R, '_categorize')
+    def test_set_metric_with_category(self, mock_categorize):
+        """Test setting metrics using ``R.set_metric`` when a category
+        is provided."""
+
+        # get the metric keys so we can check for the appropriate calls
+        self.r.set_metric('test-slug', 42, category='Test Category')
+        mock_categorize.assert_called_once_with('test-slug', "Test Category")
+
     def test_metric(self):
         """Test setting metrics using ``R.metric``."""
 
@@ -252,13 +393,17 @@ class TestR(TestCase):
 
         # get the keys used for the metric, so we can check for the appropriate
         # calls
-        day, week, month, year = self.r._build_keys(slug)
+        keys = self.r._build_keys(slug)
+        second, minute, hour, day, week, month, year = keys
         self.r.metric(slug, num=n)
 
         # Verify that setting a metric adds the appropriate slugs to the keys
         # set and then incrememts each key
         self.redis.assert_has_calls([
-            call.sadd(self.r._metric_slugs_key, day, week, month, year),
+            call.sadd(self.r._metric_slugs_key, slug),
+            call.incr(second, n),
+            call.incr(minute, n),
+            call.incr(hour, n),
             call.incr(day, n),
             call.incr(week, n),
             call.incr(month, n),
@@ -277,13 +422,17 @@ class TestR(TestCase):
         n = 1
 
         # get the keys used for the metric, so we can check for calls
-        day, week, month, year = self.r._build_keys(slug)
+        keys = self.r._build_keys(slug)
+        second, minute, hour, day, week, month, year = keys
         self.r.metric(slug, num=n, category=category)
 
         # Verify that setting a metric adds the appropriate slugs to the keys
         # set and then incrememts each key
         self.redis.assert_has_calls([
-            call.sadd(self.r._metric_slugs_key, day, week, month, year),
+            call.sadd(self.r._metric_slugs_key, slug),
+            call.incr(second, n),
+            call.incr(minute, n),
+            call.incr(hour, n),
             call.incr(day, n),
             call.incr(week, n),
             call.incr(month, n),
@@ -305,13 +454,17 @@ class TestR(TestCase):
         n = 1
 
         # get the keys used for the metric, so we can check for calls
-        day, week, month, year = self.r._build_keys(slug)
+        keys = self.r._build_keys(slug)
+        seconds, minutes, hour, day, week, month, year = keys
         self.r.metric(slug, num=n, expire=3600)
 
         # Verify that setting a metric adds the appropriate slugs to the keys
         # set and then incrememts each key
         self.redis.assert_has_calls([
-            call.sadd(self.r._metric_slugs_key, day, week, month, year),
+            call.sadd(self.r._metric_slugs_key, slug),
+            call.incr(seconds, n),
+            call.incr(minutes, n),
+            call.incr(hour, n),
             call.incr(day, n),
             call.incr(week, n),
             call.incr(month, n),
@@ -335,8 +488,11 @@ class TestR(TestCase):
         self.r.get_metric(slug)
 
         # Verify that we GET the keys from redis
-        day, week, month, year = self.r._build_keys(slug)
+        sec, min, hour, day, week, month, year = self.r._build_keys(slug)
         self.redis.assert_has_calls([
+            call.get(sec),
+            call.get(min),
+            call.get(hour),
             call.get(day),
             call.get(week),
             call.get(month),
@@ -344,22 +500,29 @@ class TestR(TestCase):
         ])
 
     def test_get_metrics(self):
+        # Set a return value for mget, so all of the method gets exercised.
+        prev_return = self.redis.mget.return_value
+        self.redis.mget.return_value = [u'1', u'2']
 
         # Slugs for metrics we want
         slugs = ['metric-1', 'metric-2']
 
         # Build the various keys for each metric
-        keys = []
+        keys_list = []
         for s in slugs:
-            day, week, month, year = self.r._build_keys(s)
-            keys.extend([day, week, month, year])
+            keys_list.append(self.r._build_keys(s))
 
-        # construct the calls to redis
-        calls = [call.get(k) for k in keys]
+        # construct the calls to redis.mget
+        calls = [call(*keys) for keys in keys_list]
 
         # Test our method
         self.r.get_metrics(slugs)
-        self.redis.assert_has_calls(calls)
+        self.assertEqual(self.redis.mget.call_args_list, calls)
+        self.assertTrue(self.redis.mget.called)  # mget was called...
+        self.assertEqual(self.redis.mget.call_count, 2)  # ...twice
+
+        # Reset mget's previous return value
+        self.redis.mget.return_value = prev_return
 
     def test_get_category_metrics(self):
         """returns metrics for a given category"""
@@ -376,7 +539,7 @@ class TestR(TestCase):
         r_kwargs = {
             'decode_responses': True,
             'host': 'localhost',
-            'db':0,
+            'db': 0,
             'port': 6379,
             'password': None,
             'connection_pool': None,
@@ -415,7 +578,7 @@ class TestR(TestCase):
             r.reset_category("Stuff", ['foo', 'bar'])
             mock_redis.assert_has_calls([
                 call(**r_kwargs),
-                call().set('c:Stuff', '["foo", "bar"]'),
+                call().sadd('c:Stuff', 'foo', 'bar'),
                 call().sadd('categories', 'Stuff'),
             ])
 
@@ -427,7 +590,7 @@ class TestR(TestCase):
             slugs = [slugs]
         keys = set()
         for slug in slugs:
-            for date in self.r._date_range(since):
+            for date in self.r._date_range(granularity, since):
                 keys.update(set(self.r._build_keys(slug, date, granularity)))
         return keys
 
@@ -437,6 +600,10 @@ class TestR(TestCase):
         self.r.get_metric_history(slugs, granularity=granularity)
         self.redis.assert_has_calls([call.mget(keys)])
 
+    def test_get_metric_history_hourly(self):
+        """Tests ``R.get_metric_history`` with hourly granularity."""
+        self._test_get_metric_history('test-slug', 'hourly')
+
     def test_get_metric_history_daily(self):
         """Tests ``R.get_metric_history`` with daily granularity."""
         self._test_get_metric_history('test-slug', 'daily')
@@ -453,6 +620,9 @@ class TestR(TestCase):
         """Tests ``R.get_metric_history`` with yearly granularity."""
         self._test_get_metric_history('test-slug', 'yearly')
 
+    def test_get_metric_multiple_history_hourly(self):
+        self._test_get_metric_history(['foo', 'bar'], 'hourly')
+
     def test_get_metric_multiple_history_daily(self):
         self._test_get_metric_history(['foo', 'bar'], 'daily')
 
@@ -465,6 +635,39 @@ class TestR(TestCase):
     def test_get_metric_multiple_history_yearly(self):
         self._test_get_metric_history(['foo', 'bar'], 'yearly')
 
+    @patch.object(R, '_date_range')
+    def test_get_metric_history_replaces_none_with_zero(self, mock_date_range):
+        """Ensure that None-values get replaced with Zeros in
+        ``R.get_metric_history``."""
+
+        # Mock the _date_ranch method so we can specify it's return values.
+        mock_date_range.return_value = [
+            datetime(2000, 1, 1),
+            datetime(2000, 1, 2),
+            datetime(2000, 1, 3),
+            datetime(2000, 1, 4),
+        ]
+
+        # Temporarily change the return value for mget
+        mget_return = self.redis.mget.return_value
+        self.redis.mget.return_value = [u'1', u'2', None, u'3']
+
+        # Note: we're not providing a since parameter here, since we've
+        # mocked the R._date_range method.
+        results = self.r.get_metric_history("foo", granularity="daily")
+
+        # Format the range of dates that for which we should get results
+        expected = [
+            ('m:foo:2000-01-01', u'1'),
+            ('m:foo:2000-01-02', u'2'),
+            ('m:foo:2000-01-03', 0),
+            ('m:foo:2000-01-04', u'3'),
+        ]
+        self.assertEqual(results, expected)
+
+        # Reset mget's previous return value
+        self.redis.mget.return_value = mget_return
+
     @patch.object(R, 'get_metric_history')
     def test_get_metric_history_as_columns(self, mock_metric_hist):
         # set up some sample (yearly) metrics
@@ -510,10 +713,14 @@ class TestR(TestCase):
             r = mock_r.return_value  # Get an instance of our Mocked R class
             r.get_metric_history_as_columns(slugs, granularity=granularity)
             mock_r.assert_has_calls([
-                call().get_metric_history_as_columns(slugs,
-                                                     granularity=granularity)
+                call().get_metric_history_as_columns(
+                    slugs, granularity=granularity
+                )
             ])
 
+    def test_get_metric_history_as_columns_hourly(self):
+        self._test_get_metric_history_as_columns(['foo', 'bar'], 'hourly')
+
     def test_get_metric_history_as_columns_daily(self):
         self._test_get_metric_history_as_columns(['foo', 'bar'], 'daily')
 
@@ -526,6 +733,32 @@ class TestR(TestCase):
     def test_get_metric_history_as_columns_yearly(self):
         self._test_get_metric_history_as_columns(['foo', 'bar'], 'yearly')
 
+    @patch.object(R, 'get_metric_history')
+    def test_get_metric_history_chart_data(self, mock_metric_hist):
+        # set up some sample (yearly) metrics
+        mock_metric_hist.return_value = [
+            ("m:bar:y:2012", '1'),
+            ('m:bar:y:2013', '2'),
+            ('m:foo:y:2012', '3'),
+            ('m:foo:y:2013', '4'),
+        ]
+        expected_results = {
+            'periods': [u'y:2012', u'y:2013'],
+            'data': [
+                {'slug': u'bar', 'values': ['1', '2']},
+                {'slug': u'foo', 'values': ['3', '4']},
+            ]
+        }
+        with patch('redis_metrics.models.redis.StrictRedis'):
+            r = R()
+            kwargs = {
+                'slugs': ['foo', 'bar'],
+                'since': None,
+                'granularity': 'yearly',
+            }
+            results = r.get_metric_history_chart_data(**kwargs)
+            self.assertEqual(results, expected_results)
+
     def test_gauge_slugs(self):
         """Tests that ``R.gauge_slugs`` calls the SMEMBERS command."""
         self.r.gauge_slugs()
@@ -541,7 +774,7 @@ class TestR(TestCase):
         is added to the set of gauge slugs and that the value gets set."""
         self.r.gauge('test-gauge', 9000)
         self.redis.assert_has_calls([
-            call.sadd(self.r._gauge_slugs_key, 'g:test-gauge'),
+            call.sadd(self.r._gauge_slugs_key, 'test-gauge'),
             call.set('g:test-gauge', 9000),
         ])
 
@@ -556,5 +789,5 @@ class TestR(TestCase):
         self.r.delete_gauge("test-gauge")
         self.redis.assert_has_calls([
             call.delete('g:test-gauge'),
-            call.srem(self.r._gauge_slugs_key, "g:test-gauge"),
+            call.srem(self.r._gauge_slugs_key, "test-gauge"),
         ])
