@@ -34,7 +34,8 @@ class TestViews(TestCase):
             email="redis_metrics_test_user@example.com",
             password="secret"
         )
-        assert self.client.login(username="redis_metrics_test_user",
+        assert self.client.login(
+            username="redis_metrics_test_user",
             password="secret")
         self.unauthed_client = Client()  # Keep an unauthenticated client
 
@@ -102,54 +103,23 @@ class TestViews(TestCase):
         slug = u'test-metric'
         url = reverse('redis_metric_detail', args=[slug])
 
-        with patch('redis_metrics.views.R') as mock_r:
-            # Set up a return value for ``R.get_metric(slug)``
-            r = mock_r.return_value  # Get an instance of our Mocked R class
-            m = {'day': '1', 'month': '1', 'week': '1', 'year': '1'}
-            r.get_metric.return_value = m
-
-            # Do the Request & test results
-            resp = self.client.get(url)
-            self.assertEqual(resp.status_code, 200)
-            self.assertEqual(resp.context_data['slug'], slug)
-            self.assertEqual(resp.context_data['metrics'], m)
-
-            # Make sure our Mocked R instance had its ``get_metric`` method
-            # called with the correct parameter
-            r.assert_has_calls([call.get_metric(slug)])
+        # Do the Request & test results
+        resp = self.client.get(url)
+        self.assertEqual(resp.status_code, 200)
+        self.assertEqual(resp.context_data['slug'], slug)
 
     def test_metric_history(self):
         slug = u'test-metric'
         granularity = u'daily'
         url = reverse('redis_metric_history', args=[slug, granularity])
 
-        with patch('redis_metrics.views.R') as mock_r:
-            # Set up a return value for ``R
-            r = mock_r.return_value  # Get an instance of our Mocked R class
-            mocked_history = [
-                ('m:{0}:2012-12-26'.format(slug), None),
-                ('m:{0}:2012-12-27'.format(slug), None),
-                ('m:{0}:2012-12-28'.format(slug), '1'),
-            ]
-            r.get_metric_history.return_value = mocked_history
-
-            # Do the Request & test results
-            resp = self.client.get(url)
-            self.assertEqual(resp.status_code, 200)
-            context = resp.context_data
-            self.assertEqual(context['slug'], slug)
-            self.assertEqual(context['granularity'], granularity)
-            self.assertEqual(context['metric_history'], mocked_history)
-
-            # Make sure our Mocked R instance had its ``get_metric_history``
-            # method called with the correct parameters
-            r.assert_has_calls([
-                call.get_metric_history(
-                    since=None,
-                    slugs=slug,
-                    granularity=granularity
-                )
-            ])
+        # Do the Request & test results
+        resp = self.client.get(url)
+        self.assertEqual(resp.status_code, 200)
+        context = resp.context_data
+        self.assertEqual(context['slug'], slug)
+        self.assertEqual(context['since'], None)
+        self.assertEqual(context['granularity'], granularity)
 
     def test_metric_history_since(self):
         """Tests the ``MetricHistoryView`` when there's a ``since`` variable"""
@@ -157,51 +127,21 @@ class TestViews(TestCase):
         granularity = u'daily'
         url = reverse('redis_metric_history', args=[slug, granularity])
 
-        with patch('redis_metrics.views.R') as mock_r:
-            # Set up a return value for ``R
-            r = mock_r.return_value  # Get an instance of our Mocked R class
-            mocked_history = [
-                ('m:{0}:2012-12-26'.format(slug), None),
-                ('m:{0}:2012-12-27'.format(slug), None),
-                ('m:{0}:2012-12-28'.format(slug), '1'),
-            ]
-            r.get_metric_history.return_value = mocked_history
+        # Do the Request & test results for a Date
+        resp = self.client.get(url, {'since': '2012-12-25'})
+        self.assertEqual(resp.status_code, 200)
+        context = resp.context_data
+        self.assertEqual(context['since'], datetime(2012, 12, 25))
+        self.assertEqual(context['slug'], slug)
+        self.assertEqual(context['granularity'], granularity)
 
-            # Do the Request & test results
-            resp = self.client.get(url, {'since': '2012-12-25'})
-            self.assertEqual(resp.status_code, 200)
-            context = resp.context_data
-            self.assertEqual(context['slug'], slug)
-            self.assertEqual(context['granularity'], granularity)
-            self.assertEqual(context['metric_history'], mocked_history)
-
-            # Make sure our Mocked R instance had its ``get_metric_history``
-            # method called with the correct parameters
-            r.assert_has_calls([
-                call.get_metric_history(
-                    since=datetime(2012, 12, 25, 0, 0),
-                    slugs=slug,
-                    granularity=granularity
-                )
-            ])
-
-    def test_metric_history_raises_keyerror(self):
-        """Tests the ``MetricHistoryView`` when there's a ``since`` variable"""
-        slug = u'test-metric'
-        granularity = u'daily'
-        url = reverse('redis_metric_history', args=[slug, granularity])
-
-        with patch('redis_metrics.views.R') as mock_r:
-            # there are a couple ways we could get a KeyError (e.g., invalid
-            # kwargs passed into ``get_context_data``, but it's convenient to
-            # test that ``get_metric_history`` raises this, since we're already
-            # mocking it.
-            r = mock_r.return_value
-            r.get_metric_history.side_effect = KeyError
-
-            # Do the Request & test results
-            resp = self.client.get(url)
-            self.assertEqual(resp.status_code, 404)
+        # Do the Request & test results for a Date & Time
+        resp = self.client.get(url, {'since': '2012-12-25 11:30:45'})
+        self.assertEqual(resp.status_code, 200)
+        context = resp.context_data
+        self.assertEqual(context['since'], datetime(2012, 12, 25, 11, 30, 45))
+        self.assertEqual(context['slug'], slug)
+        self.assertEqual(context['granularity'], granularity)
 
     def test_metrics_list_requires_admin(self):
         """Verifies that ``MetricsListView`` requires authentication."""
@@ -261,81 +201,32 @@ class TestViews(TestCase):
         slugs = '+'.join(slug_set)
         url = reverse('redis_metric_aggregate_detail', args=[slugs])
 
-        with patch('redis_metrics.views.R') as mock_r:
-            # Set up a return value for ``R.get_metric(slug)``
-            r = mock_r.return_value  # Get an instance of our Mocked R class
-            metric_list = []
-            for slug in slug_set:
-                metric_list.append((slug, {
-                        'day': '1',
-                        'month': '22',
-                        'week': '333',
-                        'year': '4444'
-                    })
-                )
-            r.get_metric.return_value = metric_list
+        # Do the Request & test results
+        resp = self.client.get(url)
+        self.assertEqual(resp.status_code, 200)
+        self.assertIn('slugs', resp.context_data)
+        self.assertEqual(resp.context_data['slugs'], slug_set)
 
-            # Do the Request & test results
-            resp = self.client.get(url)
-            self.assertEqual(resp.status_code, 200)
-            self.assertIn('slugs', resp.context_data)
-            self.assertIn('metrics', resp.context_data)
-            self.assertEqual(resp.context_data['slugs'], slug_set)
-
-            # Make sure our Mocked R instance had its ``get_metrics`` method
-            # called with the correct parameter
-            r.assert_has_calls([call.get_metrics(slug_set)])
-
-    def _metrichistory(self, slugs, granularity):
-        """Create an appropriate return value for ``R.get_metric_history``
-        based on the given slugs and granularity."""
-        history = []
-        value = 0
-        for slug in slugs:
-            if granularity == "daily":
-                key_pattern = "m:{0}:2013-01-10"
-            elif granularity == "weekly":
-                key_pattern = "m:{0}:w:2013-01"
-            elif granularity == "monthly":
-                key_pattern = "m:{0}:m:2013-01"
-            elif granularity == "yearly":
-                key_pattern = "m:{0}:y:2013"
-
-            history.append((key_pattern.format(slug), value))
-            value += 1
-        return history
-
-    def _test_aggregate_history_view(self, slugs, granularity, since=None):
-        """Tests ``views.AggregateHistoryView`` with the given slugs and
-        granularity (i.e. 'daily', 'weekly', 'monthly', 'yearly')."""
+    def _test_aggregate_history_view(self, slugs, granularity):
+        """Tests ``views.AggregateHistoryView`` with the given slugs,
+        granularity, without a specified date verifying the values are
+        correctly passed to the template."""
         slug_set = set(slugs)
         url = reverse('redis_metric_aggregate_history',
                       args=['+'.join(slugs), granularity])
 
-        with patch('redis_metrics.views.R') as mock_r:
-            # Set up a return value for ``get_metric_history_as_columns``
-            r = mock_r.return_value  # Get an instance of our Mocked R class
-            r.get_metric_history.return_value = self._metrichistory(
-                slugs, granularity)
+        # Do the Request & test results
+        resp = self.client.get(url)
+        self.assertEqual(resp.status_code, 200)
+        self.assertIn('slugs', resp.context_data)
+        self.assertIn('granularity', resp.context_data)
+        self.assertIn('since', resp.context_data)
+        self.assertEqual(resp.context_data['slugs'], slug_set)
+        self.assertEqual(resp.context_data['granularity'], granularity)
+        self.assertIsNone(resp.context_data['since'])
 
-            # Do the Request & test results
-            resp = self.client.get(url)
-            self.assertEqual(resp.status_code, 200)
-            self.assertIn('slugs', resp.context_data)
-            self.assertIn('granularity', resp.context_data)
-            self.assertIn('metric_history', resp.context_data)
-            self.assertEqual(resp.context_data['slugs'], slug_set)
-            self.assertEqual(resp.context_data['granularity'], granularity)
-
-            # Make sure our Mocked R instance had its
-            # ``get_metric_history_as_columns`` method called with the correct
-            # parameters
-            c = call.get_metric_history_as_columns(
-                slugs=slugs,
-                granularity=granularity,
-                since=since
-            )
-            r.assert_has_calls([c])
+    def test_aggregate_history_view_hourly(self):
+        self._test_aggregate_history_view(['foo', 'bar'], 'hourly')
 
     def test_aggregate_history_view_daily(self):
         self._test_aggregate_history_view(['foo', 'bar'], 'daily')
@@ -353,49 +244,25 @@ class TestViews(TestCase):
         """Tests ``views.AggregateHistoryView`` with a ``since`` parameter."""
         slugs = ['foo', 'bar']
         granularity = 'yearly'
-        url = reverse('redis_metric_aggregate_history',
-                      args=['+'.join(slugs), granularity])
+        url = reverse(
+            'redis_metric_aggregate_history',
+            args=['+'.join(slugs), granularity]
+        )
 
-        with patch('redis_metrics.views.R') as mock_r:
-            # Set up a return value for ``get_metric_history_as_columns``
-            r = mock_r.return_value  # Get an instance of our Mocked R class
-            r.get_metric_history.return_value = self._metrichistory(
-                slugs, granularity)
+        # Do the Request & test results with a just a date
+        resp = self.client.get(url, {'since': "2012-12-25"})
+        self.assertEqual(resp.status_code, 200)
+        self.assertIsInstance(resp.context_data['since'], datetime)
+        self.assertEqual(resp.context_data['since'], datetime(2012, 12, 25))
 
-            # Do the Request & test results
-            resp = self.client.get(url, {'since': "2012-12-25"})
-            self.assertEqual(resp.status_code, 200)
-
-            # Make sure our Mocked R instance had its
-            # ``get_metric_history_as_columns`` method called with the correct
-            # parameters
-            c = call.get_metric_history_as_columns(
-                slugs=slugs,
-                granularity=granularity,
-                since=datetime(2012, 12, 25)
-            )
-            r.assert_has_calls([c])
-
-    def test_aggregate_history_view_raises_keyerror(self):
-        """Tests ``views.AggregateHistoryView`` raises a 404 if
-        there's a KeyError."""
-
-        slugs = ['foo', 'bar']
-        granularity = 'yearly'
-        url = reverse('redis_metric_aggregate_history',
-                      args=['+'.join(slugs), granularity])
-
-        with patch('redis_metrics.views.R') as mock_r:
-            # there are a couple ways we could get a KeyError (e.g., invalid
-            # kwargs passed into ``get_context_data``, but it's convenient to
-            # test that ``get_metric_history`` raises this, since we're already
-            # mocking it.
-            r = mock_r.return_value  # Get an instance of our Mocked R class
-            r.get_metric_history_as_columns.side_effect = KeyError
-
-            # Do the Request & test results
-            resp = self.client.get(url)
-            self.assertEqual(resp.status_code, 404)
+        # Do the Request & test results with a date & time
+        resp = self.client.get(url, {'since': "2012-12-25 11:30:45"})
+        self.assertEqual(resp.status_code, 200)
+        self.assertIsInstance(resp.context_data['since'], datetime)
+        self.assertEqual(
+            resp.context_data['since'],
+            datetime(2012, 12, 25, 11, 30, 45)
+        )
 
     def test_category_form_view(self):
         """Verifies that GET requests to the ``CategoryFormView`` have the
